package com.hfkj.service.order.impl;

import com.hfkj.common.utils.DateUtil;
import com.hfkj.common.utils.RandomUtils;
import com.hfkj.common.utils.RedisUtil;
import com.hfkj.dao.BsOrderMapper;
import com.hfkj.entity.BsOrder;
import com.hfkj.entity.BsOrderDeduction;
import com.hfkj.entity.BsOrderExample;
import com.hfkj.model.order.OrderChildModel;
import com.hfkj.model.order.OrderModel;
import com.hfkj.service.order.BsOrderDeductionService;
import com.hfkj.service.order.BsOrderService;
import com.hfkj.sysenum.order.OrderChildProductTypeEnum;
import com.hfkj.sysenum.order.OrderStatusEnum;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.thymeleaf.util.DateUtils;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * @className: BsOrderServiceImpl
 * @author: HuRui
 * @date: 2024/4/29
 **/
@Service("orderService")
public class BsOrderServiceImpl implements BsOrderService {
    // 缓存前缀KEY
    private final static String CACHE_KEY = "ORDER:";
    // 订单缓存时间 7天
    private final static Integer CACHE_TIME = 60*60*24*7;

    @Autowired
    private RedisUtil redisUtil;
    @Resource
    private BsOrderMapper orderMapper;
    @Resource
    private BsOrderDeductionService orderDeductionService;

    @Override
    public BsOrder editData(BsOrder order) {
        order.setUpdateTime(new Date());
        if (order.getId() == null) {
            order.setCreateTime(new Date());
            orderMapper.insert(order);
        } else {
            orderMapper.updateByPrimaryKey(order);
        }
        return order;
    }

    public static void main(String[] args) {
        System.out.println(DateUtil.date2String(new Date(), "yyMMddHHmmss") + RandomUtils.number(6, false));
    }
    @Override
    @Transactional(propagation= Propagation.REQUIRES_NEW, rollbackFor= {RuntimeException.class}, timeout = 10)
    public OrderModel create(OrderModel order) {
        // 生成交易单号，共18位，时间 + 6位随机数
        order.setOrderNo(DateUtil.date2String(new Date(), "yyMMddHHmmss") + RandomUtils.number(6, false));
        order.setOrderStatus(OrderStatusEnum.status1.getCode());
        editData(order);

        // 交易优惠
        BsOrderDeduction orderDeduction = order.getDeduction()!=null?order.getDeduction():new BsOrderDeduction();
        orderDeduction.setOrderId(order.getId());
        orderDeduction.setOrderNo(order.getOrderNo());
        orderDeductionService.editData(orderDeduction);

        for (OrderChildModel child : order.getOrderChildList()) {
            child.setOrderNo(order.getOrderNo());
            // 子订单号 交易id + 8位随机数
            child.setChildOrderNo(order.getId()+RandomUtils.number(8, false));
            // 提交订单前产品处理
            if (child.getProductType().equals(OrderChildProductTypeEnum.type1.getCode())) {

            }
        }

        // 缓存
        redisUtil.set(CACHE_KEY + order.getOrderNo(), order, CACHE_TIME);
        return order;
    }

    @Override
    public BsOrder getOrder(String orderNo) {
        BsOrderExample example = new BsOrderExample();
        example.createCriteria().andOrderNoEqualTo(orderNo);
        List<BsOrder> list = orderMapper.selectByExample(example);
        if (!list.isEmpty()) {
            return list.get(0);
        }
        return null;
    }

    @Override
    public OrderModel getDetail(String orderNo) {
        Object cacheObj = redisUtil.get(CACHE_KEY + orderNo);
        if (cacheObj != null) {
            return (OrderModel) cacheObj;
        }
        OrderModel orderModel = new OrderModel();
        BeanUtils.copyProperties(getOrder(orderNo), orderModel);



        return null;
    }
}
